;zahn.lsp
; Diese Lisproutine zeichnet einen Evolventenzahnkranz auf dem Layer "ZR" als
; Polylinie, dessen Mittelpunkt dem Einfgepunkt entspricht. Der Teilkreis
; und das Zentrumkreuz werden auf dem Layer "MITTE" (sofern vorhanden) 
; erzeugt.
; Es werden die Anzahl der Zhne, das Modul, der Eingriffwinkel, das Kopfspiel
; und die Zahnfurundung abgefragt.
; Die Vorgaben in den Abfragen knnen leicht eigenen Bedrfnissen angepasst
; werden. Hierzu sind die entsprechenden "cond"-Anweisungen (direkt nach dem
; jeweiligen Abfragebefehl) zu editieren. Beispiel: In der Abfrage zum 
; Kopfspiel wird in der ihr folgenden Zeile 
; "(cond ((null c) (setq c (* 0.25 m))))" die Zahl 0.25 durch den neuen 
; Standardwert als Faktor fr das Modul m ersetzt. Desgleichen in der 
; entsprechenden Abfrage selbst.
; Die Genauigkeit bei der Konstruktion der Evolvente fr die Zahnflanke bildet
; den Winkel (N-ten Teil von 15 Grad), mit dem die Eingrifflinie auf dem
; Grundkreis abgerollt wird .
                                                                                                                         
;------------------------------------------------------------------------------
                           ;Eingabefehlerbehandlung
(defun Fehler ()
 (textscr)
 (princ "\n\n Fehler - Funktion abgebrochen")
 (princ "\n Eingriffwinkel oder Furundung zu gro\n")
 (command "layer" "se" lay "")              ;alten Layer setzen
 (setvar "blipmode" sblip)                  ;Rcksetzen der Systemvariablen
 (setvar "cmdecho" scmde)
 (quit)
 (princ)
)
;------------------------------------------------------------------------------

 (defun Fussrund1 (rhof rf winfu)
   
 ;inv_phie definieren, da dieser Wert bei der Berechnung der rechten 
 ;Zahnflanke herangezogen wird, und sich wegen der Fallunterscheidung
 ;(re < rb) vom Wert aus 'Fussrund2' (re > rb) unterscheidet.

   (setq inv_phie 0.0)

 ;sinrhofs = sin des Startwinkels winrhof der Furundung (s. trigon. Formeln) 
  
   (setq sinrhofs (/ rhof (+ rhof rf)))                  
 

 ;winrhof = tan des Startwinkels der Furundung = Bogenlnge der Furundung
 ;= Winkel im Bogenma. (sinx/cosx=tanx, mit cosx=sqrt(1-sinx))
 ;winrhof wird vom Winkel 0 aus in negativer Richtung abgetragen. 
 ;---> (- (...))
 
   (setq winrhof (- (/ sinrhofs (sqrt (- 1 (* sinrhofs sinrhofs))))))
  

 ;Radius re vom Einfgepunkt mp zum Endpunkt des Furundungbogens an der 
 ;Tangente zur Evolvente. (Das Ende des Furundungsbogens wird mit dem 
 ;Startpunkt der Evolvente auf dem Grundkreis verbunden)

   (setq re (sqrt (- (* (+ rhof rf) (+ rhof rf)) (* rhof rhof))))
 
 ;Definition der zu verbindenden Punkte (unterschiedlich zu Fussrund2):
 
 (setq p1 (polar mp winfu rf))    ;Start Fukreisbogen
 (setq p2 (polar mp winrhof rf))  ;Start Furundung
 (setq p3 (polar mp 0 re))        ;Start Tangente zur Evolvente am Grundkreis
                                  
 (setq p4 (polar mp 0 rb))        ;Start Evolvente vom Grundkreis, da
                                  ;der Fukreis kleiner als
                                  ;der Grundkreis ist.
 

 ;Zeichnen von Fukreisbogen (=Kreis "k" mit Mittelpunkt "m" in mp von p1-p2), 
 ;Furundungsbogen (Kreis "k" zwischen zwei Punkten p2-p3 und
 ;Linie "l" nach p4, als Polylinie.

 (command "plinie" p1 "kreis" "m" mp p2 p3 "l" p4)

 )                                              ;Ende Fussrund1
 
;------------------------------------------------------------------------------
(defun Fussrund2 (rhof rf rb winfu)
  
 ;Es wird der Start- und Endpunkt des Furundungsbogens in Polarkoordinaten 
 ;bezglich der 0-Achse und dem Ursprung (=Einfgepunkt) des 
 ;Polarkoordinatensystems berechnet. 
 ;Startpunkt: (winrhof / rf)
 ;Endpunkt  : (inv_phie / re)

 ;rhoe: Bogenlnge = Erzeugungsradius der Evolvente am Startpunkt, welcher 
 ;      der Endpunkt des Furundungsbogens ist.
  
  (setq rhoe (- (sqrt (- (* (+ rf rhof) (+ rf rhof)) (* rb rb))) rhof))
 
 ;re  : Startabstand des ersten Evolventenpunktes vom Einfgepunkt 
 ;      (= Endpunkt des Furundungsbogens.

  (setq re (sqrt (+ (* rhoe rhoe) (* rb rb))))

 ;inv_phie: Endwinkel des Furundungsradius 
  
  (setq inv_phie (- (/ rhoe rb) (atan (/ rhoe rb))))

 ;Startwinkel winrhof der Furundung auf dem Fukreis
  
  (setq winrhof (- (/ rhoe rb) (atan (/ (+ rhoe rhof)) rb)))

 ;Definition der zu verbindenden Punkte:

  (setq p1 (polar mp winfu rf))    ;Start Fukreisbogen
  (setq p2 (polar mp winrhof rf))  ;Start Furundung
                                  
                                   ;Start Evolvente vom Schnittpunkt des 
                                   ;Furundungsbogens mit der Evolvente 
  (setq p3 (polar mp inv_phie re)) 

 ;Zeichnen von Fukreisbogen (=Kreis "k" mit Mittelpunkt "m" in mp von p1-p2), 
 ;Furundungsbogen (Kreis "k" zwischen zwei Punkten p2-p3 

 ;Wenn kein Furundungsradius eingegeben wurde, dann wird die Polylinie 
 ;zwischen dem Ende des Fukreisbogens und dem Startpunkt der Evolvente im 
 ;Linienmodus gezeichnet, ansonsten im Bogenmodus. 

  (if (= rhof 0)
    (command "plinie" p1 "kreis" "m" mp p2 "l" p3)
    (command "plinie" p1 "kreis" "m" mp p2 p3)  
  )
 

)                                               ;Ende Fussrund2
;------------------------------------------------------------------------------
(defun C:ZAHN ()

 (initget 1)
 (setq mp (getpoint "\n Einfgepunkt : "))
 (initget (+ 1 2 4))
 (setq z (getint "\n Zhnezahl : "))
 (initget (+ 1 2 4))
 (setq m (getreal "\n Modul : "))
 (initget (+ 2 4))
 (setq egwin (getreal "\n Eingriffswinkel <20> : "))
 (cond ((null egwin) (setq egwin 20.0)))
 (initget 4)
 (setq c (getreal "\n Kopfspiel <0.25m> : "))
 (cond ((null c) (setq c (* 0.25 m))))
 (initget 4)
 (setq rhof (getreal "\n Zahnfurundung <0.38m> : "))
 (cond ((null rhof) (setq rhof (* 0.38 m))))
 (initget (+ 2 4))
 (setq genau (getreal "\n Genauigkeit der Evolvente fr die Zahnflanke <3>: "))
 (cond ((null genau) (setq genau 3.0)))
 (initget 1 "j n")
 (setq zenkr (getkword "\n Zentrumkreuz und Teilkreis zeichnen J/N :"))
 (if (or (= zenkr "j") (= zenkr "J"))
  (list
   (setq uel (getreal "\n berlnge des Zentrumkreuzes <5> : "))
   (cond ((null uel) (setq uel 5)))
  )
 )
;------------------------------------------------------------------------------
 (setq scmde (getvar "cmdecho"))               ;sichern Systemvariablen
 (setq sblip (getvar "blipmode"))
 (setq lay (getvar "clayer"))                  ;aktuellen Layer sichern
 (setvar "cmdecho" 0)                          ;ndern Variablen
 (setvar "blipmode" 0)
 (command "layer" "m" "zr" "")                 ;separater Layer fr Zahnkranz
;------------------------------------------------------------------------------
                         ;Bestimmungsgren
 (setq dt (* z m))                             ;Teikreisdurchmesser
 (setq rt (/ dt 2.0))
 (setq df (- dt (* 2.0 (+ m c))))              ;Fukreisdurchmesser                 
 (setq rf (/ df 2.0))
 (setq da (+ dt (* 2.0 m)))                    ;Kopfkreisdurchmesser
 (setq ra (/ da 2.0))
 (setq db (* (cos (/ (* egwin pi) 180.0)) dt)) ;Grundkreisdurchmesser
 (setq rb (/ db 2.0))
 (setq p (* m pi))                             ;Teilung
 ;----------------------------------------------------------------------------
                         ;zeichnen Kopfkreisbogen
 
 
 ;Bogenlnge UT zwischen dem Schnittpunkt U (siehe DIN-Normen) der Zahnflanke 
 ;mit dem Grundkreis (db) und dem Schnittpunkt T der Eingriffslinie g mit dem 
 ;Grundkreis (db). 
 ;Dies entspricht gleichzeitig der geraden Strecke C-T (=rho) der 
 ;Eingriffslinie g vom Wlzpunkt C zum Schnittpunkt T mit dem Grundkreis (db).

 (setq rho (sqrt (- (* rt rt) (* rb rb))))      

 ;Evolventenfunktion inv_alpha
 ;rho/rb = tan(alpha),  atan(rho/rb) = alpha !
 
 (setq inv_alpha (- (/ rho rb) (atan (/ rho rb))))
 
 ;Bogenlnge U'T' zwischen dem Schnittpunkt U der Zahnflanke mit dem 
 ;Grundkreis (db) und dem Schnittpunkt T der Tangente zwischen einem 
 ;beliebigen Punkt Y auf der Evolvente (Zahnflanke) mit dem Grundkreis (db). 
 ;Dies entspricht gleichzeitig der geraden Strecke YT (Erzeugungsradius fr
 ;die Evolvente im Punkt Y = ErzradY) von Y zum Schnittpunkt T mit dem 
 ;Grundkreis (db).
 ;Hier interressiert das Ende der Evolvente (Y) am Kopfkreisbogen.

 (setq ErzradY (sqrt (- (* ra ra) (* rb rb))))

 ;Differenzwinkel inv_delta zwischen der Geraden ra (=Kopfkreisradius) durch 
 ;YT und der Geraden rb (=Grundkreisradius) durch den Anfangspunkt U der 
 ;Evolvente am Grundkreis. (=Startwinkel Kopfkreisbogen)
 
 (setq inv_delta (- (/ ErzradY rb) (atan (/ ErzradY rb))))
 
 ;Winkel der Spiegelachse. 
 ;Teilungswinkel * Teilkreisradius = Teilung p (Winkel im Bogenma). 
 ;Teilungswinkel = Teilung p (Bogenma) / Teilkreisradius.
 ;Der Teilkreis wird jeweils nach p/2 von den Evolventen im Wlzpunkt C 
 ;geschnitten. Die Spiegelachse (=Symmetrieachse des Zahnes) liegt von C aus 
 ;also p/4 entfernt. 
 ;Vom Startpunkt U der Evolvente auf dem Grundkreis liegt die Spiegelachse 
 ;ein Viertel der Teilung p + inv_alpha entfernt.

 (setq winsp (+ (/ p rt 4.0) inv_alpha))            
 
 ;Endwinkel Kopfkreisbogen bezglich dem Anfangspunkt U der Evolvente auf dem
 ;Grundkreis.

 (setq winko (+ inv_delta (* (- winsp inv_delta) 2.0)))

 ;Zeichnen eines Bogens mit dem Mittelpunkt "m" in mp vom in Polarkoordinaten
 ;mit Ursprung in mp definierten Punkt mit dem Winkel inv_delta und dem Radius
 ;ra (=Startpunkt des Kopfkreisbogens) zum Punkt (in Polarkoordinaten um mp): 
 ;(winko,ra).
  
 (command "bogen" "m" mp (polar mp inv_delta ra) (polar mp winko ra))
 (setq kobog (entlast))                         ;sichern Kopfkreisbogen
;-------------------------------------------------------------------------------
                  
                  ;Berechnen der Endpunkte des Fukreisbogens
 ;Startwinkel winfu des Fukreisbogens bezglich des Wlzpunktes C.
 ;Der Startwinkel des Fukreisbogens liegt um p/4 vor dem Wlzpunkt C. (s.o.)
 ;auf dem Teilkreis.

 (setq winfu (- inv_alpha (/ p rt 4.0))) 

 ;Fallunterscheidung: verschiedene Funktionen, je nach Gre des 
 ;Fukreisbogens und des radius des ersten Evolventenpunktes.

 (if (< (+ rf rhof) rb) (Fussrund1 rhof rf winfu) (Fussrund2 rhof rf rb winfu))
 
 

;------------------------------------------------------------------------------
                  ;berechnen und zeichnen rechte Zahnflanke
 
 (setq inv_beta 0.0)                    ;Differenzwinkel involut der 
                                        ;Laufvariablen beta
 (setq s re)                            ;Laufvariable s: Radius von mp zum 
                                        ;augenblicklich berechneten Evolven-
                                        ;tenpunkt. (entspricht radius re des 
                                        ;ersten Evolventenpunktes fr re > rb.
 
 ;Wenn der Radius des Anfangspunktes der Evolventenlinie re kleiner als der 
 ;Grundkreisradius ist, dann wird die Laufvariable beta (im Gradma) von 0 
 ;weg gezhlt. Im Gradma dehalb, da in der folgenden Schleife 
 ;in's Bogenma umgerechnet wird.
 ;Sonst ist beta der Winkel, der sich ergibt, wenn die Evolventenlinie 
 ;den Punkt durchluft, welcher der Endpunkt des Furundungsradius in re ist.
 
 (if (< re rb)
   (setq beta (- (/ (* 0.0 180) pi) (/ 15.0 genau)))     ;Gradma 
   (setq beta (/ (* (/ rhoe rb) 180) pi))
 )                                      ;Startwert der Laufvariablen beta als 
                                        ;Schrittwinkel zum momentan 
                                        ;berechneten Evolventenpunkt
                                        ;(im Gradma).
 
 ;Berechnung der Evolventenlinie

 ;Solange s < Kopfkreisradius ra, 

 (while (< s ra)                  ;Schleifenanfang fr Evolventenpunkte
   (setq pn                       ;Def. nchster Punkt der Evolvente Anfang
    (polar mp                     ;Polardefinition nchsten Punkt Beginn
                                  

 ;Selbe Problematik wie bei inv_alpha oder inv_delta.
 ;Es wird eine Laufvariable beta definiert, welche bei jedem Schleifen-
 ;durchlauf um 15.0/genau erhht wird. 

 ;Der Winkel inv_beta ist die Differenz zwischen dem Winkel 
 ;((beta+15/genau)*pi/180) im Bogenma und beta, welcher sich durch die 
 ;Umkehrung der Tangensfunktion (arctan(b/rb) ergibt. 
 
 ;Unter diesem Winkel (inv_beta) wird ein Evolventenpunkt mit dem Abstand s
 ;von mp berechnet.
 ;Erreicht s ra, dann wird ein letzter Punkt berechnet und die Schleife 
 ;beendet.
 
                                  ;wenn inv_beta > inv_delta...

     (if (> (setq inv_beta (- (* pi (/ (setq beta (+ beta (/ 15.0 genau))
              ) 180.0)
             ) 
            (atan (/ (setq b (/ (* pi beta rb) 180.0)) rb)))
           ) inv_delta)
      (setq inv_beta inv_delta) inv_beta ;...dann inv_beta=inv_delta setzen, 
                                  ;sonst mit inv_beta fortfahren.
     )                            
     (if (> (setq s (sqrt (+ (* rb rb) (* b b)))) ra)       ;wenn s>ra...
       (setq s ra) s)             ;..dann s=ra setzen, sonst mit s fortfahren
    
    
    )                             ;Polardefinition fr nchsten Punkt Ende
   )                              ;Def. nchster Punkt der Evolvente Ende
   (command pn)                   ;zeichnen nchsten Punkt 
 )                                ;Ende Schleife
 (command "")
 (setq rzfl (entlast))            ;rechte Zahnflanke sichern
;-------------------------------------------------------------------------------
                             ;Zahnkranz zeichnen
 ;spiegeln re. Zahnflanke
 (command "spiegeln" rzfl "" mp (polar mp winsp rt) "") 
 (setq lzfl (entlast))                         ;linke Zahnflanke sichern
 (command "pedit" rzfl "v" kobog lzfl  "" "")  ;Polylinie Zahn kompl. bilden
 (setq zahn (entlast))                         ;Sichern Zahn
 (command "reihe" zahn "" "p" mp z "" "")      ;Reihe mit Anzahl z

 ;Polylinie Zahnrad

 (command "pedit" zahn "v" (ssget "x" '((8 . "zr"))) "" "")  
 ;-----------------------------------------------------------------------------
                        ;zeichnen Teilkreis und Zentrumkreuz
 (if (= zenkr "j")
  (list 
   (command "layer" "se" "mitte" "")           ;Layer "Mitte" setzen
   (command "kreis" mp rt "")                  ;Kreis mit Mittelpunkt mp und
                                               ;Radius rt zeichnen.
   (command "linie" (polar mp winsp (+ ra uel));Erste achse um den Winkel der 
                                               ;Spiegelachse gedreht mit 
                                               ;berlnge uel zeichnen.

                                               ;Zweite Hlfte der ersten Achse
                                               ;zeichnen.
                    (polar mp (+ pi winsp) (+ ra uel)) "")
                                               
                                               ;Erste und zweite Hlfte der 
                                               ;zweiten Achse zeichnen.
   (command "linie" (polar mp (+ (/ pi 2.0) winsp) (+ ra uel)) 
                    (polar mp (+ pi (/ pi 2.0) winsp) (+ ra uel)) "")
  )
 )
;-------------------------------------------------------------------------------
 (command "layer" "se" lay "")                              ;alten Layer setzen

 (setq ausg (getstring "Bestimmungsgren ausgeben J/<N> : "))
 (if (or (= ausg "j") (= ausg "J"))
  (list (textscr)
   (list (princ "\n Anzahl der Zhne     : ") (princ z))
   (list (princ "\n Modul                : ") (princ m))
   (list (princ "\n Teilung              : ") (princ p))
   (list (princ "\n Teilkreisdurchmesser : ") (princ dt))
   (list (princ "\n Kopfkreisdurchmesser : ") (princ da))
   (list (princ "\n Fukreisdurchmesser  : ") (princ df))  
   (list (princ "\n Grundkreisdurchmesser: ") (princ db))
   (list (princ "\n Kopfspiel            : ") (princ c))
   (list (princ "\n Eingriffwinkel       : ") (princ egwin))
   (list (princ "\n Zahnfurundung       : ") (princ rhof))
  )
 )
 (setvar "cmdecho" scmde)                       ;Rcksetzen der Systemvariablen
 (setvar "blipmode" sblip)
 (princ)
)                                                                         ;Ende
